from typing import Sequence

def add_uppercase_char(char_list: Sequence[tuple[str, str]]) -> Sequence[tuple[str, str]]: ...

CYRILLIC: Sequence[tuple[str, str]]
GERMAN: Sequence[tuple[str, str]]
GREEK: Sequence[tuple[str, str]]
PRE_TRANSLATIONS: Sequence[tuple[str, str]]
