from typing import Any

change_table: Any

def modify_operation(
    dn, changes, auto_encode, schema: Any | None = ..., validator: Any | None = ..., check_names: bool = ...
): ...
def modify_request_to_dict(request): ...
def modify_response_to_dict(response): ...
