from typing import Any

class DesiredCapabilities:
    FIREFOX: Any
    INTERNETEXPLORER: Any
    EDGE: Any
    CHROME: Any
    OPERA: Any
    SAFARI: Any
    HTMLUNIT: Any
    HTMLUNITWITHJS: Any
    IPHONE: Any
    IPAD: Any
    ANDROID: Any
    PHANTOMJS: Any
    WEBKITGTK: Any
