from typing import Any

proxies: Any

def manage(module, **params): ...
def clear_managers() -> None: ...

class _DBProxy:
    module: Any
    kw: Any
    poolclass: Any
    pools: Any
    def __init__(self, module, poolclass=..., **kw) -> None: ...
    def close(self) -> None: ...
    def __del__(self) -> None: ...
    def __getattr__(self, key): ...
    def get_pool(self, *args, **kw): ...
    def connect(self, *args, **kw): ...
    def dispose(self, *args, **kw) -> None: ...
