from typing import NoReturn

def isPath(f: object) -> bool: ...
def isDirectory(f: object) -> bool: ...

class deferred_error:
    ex: BaseException
    def __init__(self, ex: BaseException) -> None: ...
    def __getattr__(self, elt: str) -> NoReturn: ...
