from braintree.util.generator import Generator as Generator
from braintree.util.parser import Parser as Parser

class XmlUtil:
    @staticmethod
    def xml_from_dict(dict): ...
    @staticmethod
    def dict_from_xml(xml): ...
