from typing import Any, Iterable, overload

class LazyFixture:
    name: str
    def __init__(self, name: str) -> None: ...
    def __eq__(self, other: object) -> bool: ...

@overload
def lazy_fixture(names: str) -> LazyFixture: ...
@overload
def lazy_fixture(names: Iterable[str]) -> list[LazyFixture] | Any: ...
def is_lazy_fixture(val: Any) -> bool: ...
def pytest_configure() -> None: ...
def __getattr__(name: str) -> Any: ...  # incomplete
