from typing import Iterable

PREFIXES: list[str]
ENABLE_USER_SITE: bool | None
USER_SITE: str | None
USER_BASE: str | None

def main() -> None: ...
def addsitedir(sitedir: str, known_paths: Iterable[str] | None = ...) -> None: ...
def getsitepackages(prefixes: Iterable[str] | None = ...) -> list[str]: ...
def getuserbase() -> str: ...
def getusersitepackages() -> str: ...
