import sys

from _operator import *

if sys.version_info >= (3, 11):
    __all__ = [
        "abs",
        "add",
        "and_",
        "attrgetter",
        "call",
        "concat",
        "contains",
        "countOf",
        "delitem",
        "eq",
        "floordiv",
        "ge",
        "getitem",
        "gt",
        "iadd",
        "iand",
        "iconcat",
        "ifloordiv",
        "ilshift",
        "imatmul",
        "imod",
        "imul",
        "index",
        "indexOf",
        "inv",
        "invert",
        "ior",
        "ipow",
        "irshift",
        "is_",
        "is_not",
        "isub",
        "itemgetter",
        "itruediv",
        "ixor",
        "le",
        "length_hint",
        "lshift",
        "lt",
        "matmul",
        "methodcaller",
        "mod",
        "mul",
        "ne",
        "neg",
        "not_",
        "or_",
        "pos",
        "pow",
        "rshift",
        "setitem",
        "sub",
        "truediv",
        "truth",
        "xor",
    ]
else:
    __all__ = [
        "abs",
        "add",
        "and_",
        "attrgetter",
        "concat",
        "contains",
        "countOf",
        "delitem",
        "eq",
        "floordiv",
        "ge",
        "getitem",
        "gt",
        "iadd",
        "iand",
        "iconcat",
        "ifloordiv",
        "ilshift",
        "imatmul",
        "imod",
        "imul",
        "index",
        "indexOf",
        "inv",
        "invert",
        "ior",
        "ipow",
        "irshift",
        "is_",
        "is_not",
        "isub",
        "itemgetter",
        "itruediv",
        "ixor",
        "le",
        "length_hint",
        "lshift",
        "lt",
        "matmul",
        "methodcaller",
        "mod",
        "mul",
        "ne",
        "neg",
        "not_",
        "or_",
        "pos",
        "pow",
        "rshift",
        "setitem",
        "sub",
        "truediv",
        "truth",
        "xor",
    ]

__lt__ = lt
__le__ = le
__eq__ = eq
__ne__ = ne
__ge__ = ge
__gt__ = gt
__not__ = not_
__abs__ = abs
__add__ = add
__and__ = and_
__floordiv__ = floordiv
__index__ = index
__inv__ = inv
__invert__ = invert
__lshift__ = lshift
__mod__ = mod
__mul__ = mul
__matmul__ = matmul
__neg__ = neg
__or__ = or_
__pos__ = pos
__pow__ = pow
__rshift__ = rshift
__sub__ = sub
__truediv__ = truediv
__xor__ = xor
__concat__ = concat
__contains__ = contains
__delitem__ = delitem
__getitem__ = getitem
__setitem__ = setitem
__iadd__ = iadd
__iand__ = iand
__iconcat__ = iconcat
__ifloordiv__ = ifloordiv
__ilshift__ = ilshift
__imod__ = imod
__imul__ = imul
__imatmul__ = imatmul
__ior__ = ior
__ipow__ = ipow
__irshift__ = irshift
__isub__ = isub
__itruediv__ = itruediv
__ixor__ = ixor
if sys.version_info >= (3, 11):
    __call__ = call
