from typing import Any

from passlib.crypto.digest import norm_hash_name as norm_hash_name

def get_prf(name): ...
def pbkdf1(secret, salt, rounds, keylen: Any | None = ..., hash: str = ...): ...
def pbkdf2(secret, salt, rounds, keylen: Any | None = ..., prf: str = ...): ...
