from . import base

class Filter(base.Filter):
    def slider(self) -> None: ...
    def __iter__(self): ...
    def is_optional_start(self, tagname, previous, next): ...
    def is_optional_end(self, tagname, next): ...
