from typing import Any

from .context import Context as _Context

class AsyncContext(_Context):
    def __init__(self, *args, loop: Any | None = ..., use_task_factory: bool = ..., **kwargs) -> None: ...
    def clear_trace_entities(self) -> None: ...

class TaskLocalStorage:
    def __init__(self, loop: Any | None = ...) -> None: ...
    def __setattr__(self, name, value) -> None: ...
    def __getattribute__(self, item): ...
    def clear(self) -> None: ...

def task_factory(loop, coro): ...
