from collections.abc import Sequence

from ttkthemes.themed_style import ThemedStyle as ThemedStyle
from ttkthemes.themed_tk import ThemedTk as ThemedTk

# actually a list, but shouldn't be modified
THEMES: Sequence[str]
