from typing import Any

class Log:
    level: Any
    def __init__(self) -> None: ...
    def to_capabilities(self): ...

class Options:
    KEY: str
    log: Any
    def __init__(self) -> None: ...
    @property
    def binary(self): ...
    @binary.setter
    def binary(self, new_binary) -> None: ...
    @property
    def binary_location(self): ...
    @binary_location.setter
    def binary_location(self, value) -> None: ...
    @property
    def accept_insecure_certs(self): ...
    @accept_insecure_certs.setter
    def accept_insecure_certs(self, value) -> None: ...
    @property
    def capabilities(self): ...
    def set_capability(self, name, value) -> None: ...
    @property
    def preferences(self): ...
    def set_preference(self, name, value) -> None: ...
    @property
    def proxy(self): ...
    @proxy.setter
    def proxy(self, value) -> None: ...
    @property
    def profile(self): ...
    @profile.setter
    def profile(self, new_profile) -> None: ...
    @property
    def arguments(self): ...
    def add_argument(self, argument) -> None: ...
    @property
    def headless(self): ...
    @headless.setter
    def headless(self, value) -> None: ...
    def set_headless(self, headless: bool = ...) -> None: ...
    def to_capabilities(self): ...
