from typing import Any

from braintree.search import Search as Search

class DisputeSearch:
    amount_disputed: Any
    amount_won: Any
    case_number: Any
    chargeback_protection_level: Any
    customer_id: Any
    disbursement_date: Any
    effective_date: Any
    id: Any
    kind: Any
    merchant_account_id: Any
    reason: Any
    reason_code: Any
    received_date: Any
    reference_number: Any
    reply_by_date: Any
    status: Any
    transaction_id: Any
    transaction_source: Any
