from typing import Any

class Alert:
    driver: Any
    def __init__(self, driver) -> None: ...
    @property
    def text(self): ...
    def dismiss(self) -> None: ...
    def accept(self) -> None: ...
    def send_keys(self, keysToSend) -> None: ...
