def fingerprint128(__a: str) -> tuple[int, int]: ...
def fingerprint32(__a: str) -> int: ...
def fingerprint64(__a: str) -> int: ...
def hash128(__a: str) -> tuple[int, int]: ...
def hash128withseed(__a: str, __seed_low: int, __seed_high: int) -> tuple[int, int]: ...
def hash32(__a: str) -> int: ...
def hash32withseed(__a: str, __seed: int) -> int: ...
def hash64(__a: str) -> int: ...
def hash64withseed(__a: str, __seed: int) -> int: ...
