"""Too-complex program."""


v = [0, 0.1, 1j, "foo", slice(0, 3), "foo", None, type, max]
w = [0, 0.1, 1j, "foo", slice(0, 3), "foo", None, type, max]
x = [0, 0.1, 1j, "foo", slice(0, 3), "foo", None, type, max]
y = [0, 0.1, 1j, "foo", slice(0, 3), "foo", None, type, max]
z = zip(v, w, x, y)
