from typing import Any

def prepare_headers(oauth_params, headers: Any | None = ..., realm: Any | None = ...): ...
def prepare_form_encoded_body(oauth_params, body): ...
def prepare_request_uri_query(oauth_params, uri): ...
