from typing import Any

class CommandsParser:
    initialized: bool
    commands: Any
    def __init__(self, redis_connection) -> None: ...
    def initialize(self, r) -> None: ...
    def get_keys(self, redis_conn, *args): ...
