class Reporter:
    def __init__(self, warningStream, errorStream) -> None: ...
    def unexpectedError(self, filename, msg) -> None: ...
    def syntaxError(self, filename, msg, lineno, offset, text) -> None: ...
    def flake(self, message) -> None: ...
