# requests also imports urllib3 as requests.packages.urllib3, the stubs don't reflect that.

class VendorAlias:
    def __init__(self, package_names) -> None: ...
    def find_module(self, fullname, path=...): ...
    def load_module(self, name): ...
