from typing import Any

def modify_dn_operation(dn, new_relative_dn, delete_old_rdn: bool = ..., new_superior: Any | None = ...): ...
def modify_dn_request_to_dict(request): ...
def modify_dn_response_to_dict(response): ...
