from _typeshed import Self

class weekday(object):
    def __init__(self, weekday: int, n: int | None = ...) -> None: ...
    def __call__(self: Self, n: int) -> Self: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    weekday: int
    n: int
