class ParserBase(object):
    def __init__(self) -> None: ...
    def error(self, message: str) -> None: ...
    def reset(self) -> None: ...
    def getpos(self) -> tuple[int, int]: ...
    def unknown_decl(self, data: str) -> None: ...
