from typing import Any

class Keys:
    NULL: str
    CANCEL: str
    HELP: str
    BACKSPACE: str
    BACK_SPACE: Any
    TAB: str
    CLEAR: str
    RETURN: str
    ENTER: str
    SHIFT: str
    LEFT_SHIFT: Any
    CONTROL: str
    LEFT_CONTROL: Any
    ALT: str
    LEFT_ALT: Any
    PAUSE: str
    ESCAPE: str
    SPACE: str
    PAGE_UP: str
    PAGE_DOWN: str
    END: str
    HOME: str
    LEFT: str
    ARROW_LEFT: Any
    UP: str
    ARROW_UP: Any
    RIGHT: str
    ARROW_RIGHT: Any
    DOWN: str
    ARROW_DOWN: Any
    INSERT: str
    DELETE: str
    SEMICOLON: str
    EQUALS: str
    NUMPAD0: str
    NUMPAD1: str
    NUMPAD2: str
    NUMPAD3: str
    NUMPAD4: str
    NUMPAD5: str
    NUMPAD6: str
    NUMPAD7: str
    NUMPAD8: str
    NUMPAD9: str
    MULTIPLY: str
    ADD: str
    SEPARATOR: str
    SUBTRACT: str
    DECIMAL: str
    DIVIDE: str
    F1: str
    F2: str
    F3: str
    F4: str
    F5: str
    F6: str
    F7: str
    F8: str
    F9: str
    F10: str
    F11: str
    F12: str
    META: str
    COMMAND: str
