from typing import Any

def get_active_sheet(wb): ...

class WorkbookWriter:
    wb: Any
    rels: Any
    package: Any
    def __init__(self, wb) -> None: ...
    def write_properties(self) -> None: ...
    def write_worksheets(self) -> None: ...
    def write_refs(self) -> None: ...
    def write_names(self) -> None: ...
    def write_pivots(self) -> None: ...
    def write_views(self) -> None: ...
    def write(self): ...
    def write_rels(self): ...
    def write_root_rels(self): ...
