from typing import Any

PATTERN: Any

class FreshnessDateDataParser:
    def get_local_tz(self): ...
    def parse(self, date_string, settings): ...
    def get_kwargs(self, date_string): ...
    def get_date_data(self, date_string, settings: Any | None = ...): ...

freshness_date_parser: Any
