def get_unpatched(item): ...
def get_unpatched_class(cls): ...
def patch_all() -> None: ...
def patch_func(replacement, target_mod, func_name) -> None: ...
def get_unpatched_function(candidate): ...
def patch_for_msvc_specialized_compiler(): ...
