from .base import (
    get_canonical_version as get_canonical_version,
    in_stdlib as in_stdlib,
    long_versions as long_versions,
    short_versions as short_versions,
    stdlib_list as stdlib_list,
)

__version__: str
