from typing import Any

log: Any

class GrantTypeBase:
    def __getattr__(self, attr): ...
    def __setattr__(self, attr, value) -> None: ...
    def validate_authorization_request(self, request): ...
    def id_token_hash(self, value, hashfunc=...): ...
    def add_id_token(self, token, token_handler, request, nonce: Any | None = ...): ...
    def openid_authorization_validator(self, request): ...

OpenIDConnectBase = GrantTypeBase
