from typing import Any

class Collection:
    tasks: Any
    collections: Any
    default: str | None
    name: str | None
    loaded_from: Any
    auto_dash_names: bool
    def __init__(self, *args, **kwargs) -> None: ...
    @classmethod
    def from_module(cls, module, name=..., config=..., loaded_from=..., auto_dash_names=...): ...
    def add_task(self, task, name=..., aliases=..., default=...) -> None: ...
    def add_collection(self, coll, name=..., default=...) -> None: ...
    def subcollection_from_path(self, path): ...
    def task_with_config(self, name): ...
    def to_contexts(self): ...
    def subtask_name(self, collection_name, task_name): ...
    def transform(self, name): ...
    @property
    def task_names(self): ...
    def configuration(self, taskpath=...): ...
    def configure(self, options) -> None: ...
    def serialized(self): ...
