long_versions: list[str]
short_versions: list[str]

def get_canonical_version(version: str) -> str: ...
def stdlib_list(version: str | None = ...) -> list[str]: ...
def in_stdlib(module_name: str, version: str | None = ...) -> bool: ...
