from . import _LangModelType

Latin7_char_to_order_map: tuple[int, ...]
win1253_char_to_order_map: tuple[int, ...]
GreekLangModel: tuple[int, ...]
Latin7GreekModel: _LangModelType
Win1253GreekModel: _LangModelType
