from typing import Any

class AtomicCounter:
    value: Any
    def __init__(self, initial: int = ...) -> None: ...
    def increment(self, num: int = ...): ...
    def decrement(self, num: int = ...): ...
    def get_current(self): ...
    def reset(self): ...
