class _NoModuleFound(Exception): ...
class InvalidName(ValueError): ...
class ModuleNotFound(InvalidName): ...
class ObjectNotFound(InvalidName): ...

def reraise(exception, traceback) -> None: ...
def namedAny(name): ...
