def f1(x):
    pass

def f2(x):
    pass

def f3(x):
    pass

def f4(x):
    pass

def f5(x):
    pass

def f6(x):
    pass

def f7(x):
    pass

def f8(x):
    pass

def f9(x):
    pass

def f10(x):
    pass
