from typing import Any

class ReadOnlyCell:
    parent: Any
    row: Any
    column: Any
    data_type: Any
    def __init__(self, sheet, row, column, value, data_type: str = ..., style_id: int = ...) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    # defined twice in the implementation
    @property
    def coordinate(self): ...
    @property
    def column_letter(self): ...
    @property
    def style_array(self): ...
    @property
    def has_style(self): ...
    @property
    def number_format(self): ...
    @property
    def font(self): ...
    @property
    def fill(self): ...
    @property
    def border(self): ...
    @property
    def alignment(self): ...
    @property
    def protection(self): ...
    @property
    def is_date(self): ...
    @property
    def internal_value(self): ...
    @property
    def value(self): ...
    @value.setter
    def value(self, value) -> None: ...

class EmptyCell:
    value: Any
    is_date: bool
    font: Any
    border: Any
    fill: Any
    number_format: Any
    alignment: Any
    data_type: str

EMPTY_CELL: Any
